@@ -35,11 +35,13 @@ NULL
 #' @importFrom future nbrOfWorkers
 #'
 #' @references Inspired by Chung et al, Bioinformatics (2014)
+#' @concept dimensional_reduction
 #'
 #' @export
 #'
 #' @examples
 #' \dontrun{
+#' data("pbmc_small")
 #' pbmc_small = suppressWarnings(JackStraw(pbmc_small))
 #' head(JS(object = pbmc_small[['pca']], slot = 'empirical'))
 #' }
@@ -65,6 +67,10 @@ JackStraw <- function(
     my.sapply <- future_sapply
   }
   assay <- assay %||% DefaultAssay(object = object)
+  if (IsSCT(assay = object[[assay]])) {
+    stop("JackStraw cannot be run on SCTransform-normalized data.
+         Please supply a non-SCT assay.")
+  }
   if (dims > length(x = object[[reduction]])) {
     dims <- length(x = object[[reduction]])
     warning("Number of dimensions specified is greater than those available. Setting dims to ", dims, " and continuing", immediate. = TRUE)
@@ -147,6 +153,7 @@ JackStraw <- function(
 #' @param new.key name of key for new dimensional reduction
 #'
 #' @return Returns a \code{\link{Seurat}} object
+#' @concept dimensional_reduction
 #'
 #' @export
 #'
@@ -179,6 +186,7 @@ L2Dim <- function(object, reduction, new.dr = NULL, new.key = NULL) {
 #'
 #' @param object Seurat object
 #' @param \dots Additional parameters to L2Dim.
+#' @concept dimensional_reduction
 #'
 #' @export
 #'
@@ -203,10 +211,12 @@ L2CCA <- function(object, ...){
 #' at least one of the given PCs.
 #'
 #' @export
+#' @concept dimensional_reduction
 #'
 #' @seealso \code{\link{ProjectDim}} \code{\link{JackStraw}}
 #'
 #' @examples
+#' data("pbmc_small")
 #' PCASigGenes(pbmc_small, pcs.use = 1:2)
 #'
 PCASigGenes <- function(
@@ -260,8 +270,10 @@ PCASigGenes <- function(
 #' @return Returns Seurat object with the projected values
 #'
 #' @export
+#' @concept dimensional_reduction
 #'
 #' @examples
+#' data("pbmc_small")
 #' pbmc_small
 #' pbmc_small <- ProjectDim(object = pbmc_small, reduction = "pca")
 #' # Vizualize top projected genes in heatmap
@@ -307,6 +319,173 @@ ProjectDim <- function(
   return(object)
 }
 
+#' @param query.dims Dimensions (columns) to use from query
+#' @param reference.dims Dimensions (columns) to use from reference
+#' @param ... Additional parameters to \code{\link{RunUMAP}}
+#'
+#' @inheritParams FindNeighbors
+#' @inheritParams RunUMAP
+#'
+#' @rdname ProjectUMAP
+#' @concept dimensional_reduction
+#' @export
+#'
+ProjectUMAP.default <- function(
+  query,
+  query.dims = NULL,
+  reference,
+  reference.dims = NULL,
+  k.param = 20,
+  nn.method = "annoy",
+  n.trees = 50,
+  annoy.metric = "cosine",
+  l2.norm = FALSE,
+  cache.index = TRUE,
+  index = NULL,
+  neighbor.name = "query_ref.nn",
+  reduction.model,
+  ...
+) {
+  query.dims <- query.dims %||% 1:ncol(x = query)
+  reference.dims <- reference.dims %||% query.dims
+ if (length(x = reference.dims) != length(x = query.dims)) {
+    stop("Length of Reference and Query number of dimensions are not equal")
+   }
+  if (any(reference.dims > ncol(x = reference))) {
+    stop("Reference dims is larger than the number of dimensions present.", call. = FALSE)
+  }
+  if (any(query.dims > ncol(x = query))) {
+    stop("Query dims is larger than the number of dimensions present.", call. = FALSE)
+  }
+  if (length(x = Misc(object = reduction.model, slot = 'model')) == 0) {
+    stop(
+      "The provided reduction.model does not have a model stored. Please try ",
+      "running umot-learn on the object first", call. = FALSE
+    )
+  }
+  query.neighbor <- FindNeighbors(
+    object = reference[, reference.dims],
+    query = query[, query.dims],
+    k.param = k.param,
+    nn.method = nn.method,
+    n.trees = n.trees,
+    annoy.metric = annoy.metric,
+    cache.index = cache.index,
+    index = index,
+    return.neighbor = TRUE,
+    l2.norm = l2.norm
+  )
+  proj.umap <- RunUMAP(object = query.neighbor, reduction.model = reduction.model, ...)
+  return(list(proj.umap = proj.umap, query.neighbor = query.neighbor))
+}
+
+#' @rdname ProjectUMAP
+#' @concept dimensional_reduction
+#' @export
+#' @method ProjectUMAP DimReduc
+#'
+ProjectUMAP.DimReduc <- function(
+  query,
+  query.dims = NULL,
+  reference,
+  reference.dims = NULL,
+  k.param = 20,
+  nn.method = "annoy",
+  n.trees = 50,
+  annoy.metric = "cosine",
+  l2.norm = FALSE,
+  cache.index = TRUE,
+  index = NULL,
+  neighbor.name = "query_ref.nn",
+  reduction.model,
+  ...
+) {
+  proj.umap <- ProjectUMAP(
+    query = Embeddings(object = query),
+    query.dims = query.dims,
+    reference = Embeddings(object = reference),
+    reference.dims = reference.dims,
+    k.param = k.param,
+    nn.method = nn.method,
+    n.trees = 50,
+    annoy.metric = annoy.metric,
+    l2.norm = l2.norm,
+    cache.index = cache.index,
+    index = index,
+    neighbor.name = neighbor.name,
+    reduction.model = reduction.model,
+    ...
+  )
+  return(proj.umap)
+}
+
+#' @param reference Reference dataset
+#' @param query.reduction Name of reduction to use from the query for neighbor
+#' finding
+#' @param reference.reduction Name of reduction to use from the reference for
+#' neighbor finding
+#' @param neighbor.name Name to store neighbor information in the query
+#' @param reduction.name Name of projected UMAP to store in the query
+#' @param reduction.key Value for the projected UMAP key
+#' @rdname ProjectUMAP
+#' @concept dimensional_reduction
+#' @export
+#' @method ProjectUMAP Seurat
+#'
+ProjectUMAP.Seurat <- function(
+  query,
+  query.reduction,
+  query.dims = NULL,
+  reference,
+  reference.reduction,
+  reference.dims = NULL,
+  k.param = 20,
+  nn.method = "annoy",
+  n.trees = 50,
+  annoy.metric = "cosine",
+  l2.norm = FALSE,
+  cache.index = TRUE,
+  index = NULL,
+  neighbor.name = "query_ref.nn",
+  reduction.model,
+  reduction.name = "ref.umap",
+  reduction.key = "refUMAP_",
+  ...
+) {
+  if (!query.reduction %in% Reductions(object = query)) {
+    stop("The query.reduction (", query.reduction, ") is not present in the ",
+         "provided query", call. = FALSE)
+  }
+  if (!reference.reduction %in% Reductions(object = reference)) {
+    stop("The reference.reduction (", reference.reduction, ") is not present in the ",
+         "provided reference.", call. = FALSE)
+  }
+  if (!reduction.model %in% Reductions(object = reference)) {
+    stop("The reduction.model (", reduction.model, ") is not present in the ",
+         "provided reference.", call. = FALSE)
+  }
+  proj.umap <- ProjectUMAP(
+    query = query[[query.reduction]],
+    query.dims = query.dims,
+    reference = reference[[reference.reduction]],
+    reference.dims = reference.dims,
+    k.param = k.param,
+    nn.method = nn.method,
+    n.trees = n.trees,
+    annoy.metric = annoy.metric,
+    l2.norm = l2.norm,
+    cache.index = cache.index,
+    index = index,
+    neighbor.name = neighbor.name,
+    reduction.model = reference[[reduction.model]],
+    reduction.key = reduction.key,
+    ...
+  )
+  query[[reduction.name]] <- proj.umap$proj.umap
+  query[[neighbor.name]] <- proj.umap$query.neighbor
+  return(query)
+}
+
 #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 # Methods for Seurat-defined generics
 #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -320,6 +499,7 @@ ProjectDim <- function(
 #' @importFrom irlba irlba
 #'
 #' @rdname RunCCA
+#' @concept dimensional_reduction
 #' @export
 #'
 RunCCA.default <- function(
@@ -372,6 +552,7 @@ RunCCA.default <- function(
 #' set to TRUE)
 #'
 #' @rdname RunCCA
+#' @concept dimensional_reduction
 #' @export
 #' @method RunCCA Seurat
 #'
@@ -502,6 +683,7 @@ RunCCA.Seurat <- function(
 #' @importFrom ica icafast icaimax icajade
 #'
 #' @rdname RunICA
+#' @concept dimensional_reduction
 #' @export
 #' @method RunICA default
 #'
@@ -550,6 +732,7 @@ RunICA.default <- function(
 #' @param features Features to compute ICA on
 #'
 #' @rdname RunICA
+#' @concept dimensional_reduction
 #' @export
 #' @method RunICA Assay
 #'
@@ -593,6 +776,7 @@ RunICA.Assay <- function(
 #' @param reduction.name dimensional reduction name
 #'
 #' @rdname RunICA
+#' @concept dimensional_reduction
 #' @method RunICA Seurat
 #' @export
 #'
@@ -632,136 +816,6 @@ RunICA.Seurat <- function(
   return(object)
 }
 
-#' @param assay Which assay to use. If NULL, use the default assay
-#' @param n Number of singular values to compute
-#' @param reduction.key Key for dimension reduction object
-#' @param scale.max Clipping value for cell embeddings. Default (NULL) is no clipping.
-#' @param seed.use Set a random seed. By default, sets the seed to 42. Setting
-#' NULL will not set a seed.
-#' @param verbose Print messages
-#'
-#' @importFrom irlba irlba
-#'
-#' @rdname RunLSI
-#' @export
-RunLSI.default <- function(
-  object,
-  assay = NULL,
-  n = 50,
-  reduction.key = 'LSI_',
-  scale.max = NULL,
-  seed.use = 42,
-  verbose = TRUE,
-  ...
-) {
-  CheckDots(...)
-  if (!is.null(seed.use)) {
-    set.seed(seed = seed.use)
-  }
-  tf.idf <- TF.IDF(data = object, verbose = verbose)
-  tf.idf <- LogNorm(data = tf.idf, display_progress = verbose, scale_factor = 1e4)
-  colnames(x = tf.idf) <- colnames(x = object)
-  rownames(x = tf.idf) <- rownames(x = object)
-  n <- min(n, ncol(x = object) - 1)
-  if (verbose) {
-    message("Running SVD on TF-IDF matrix")
-  }
-  lsi <- irlba(A = t(tf.idf), nv = n)
-  feature.loadings <- lsi$v
-  sdev <- lsi$d / sqrt(max(1, nrow(x = object) - 1))
-  cell.embeddings <- lsi$u
-  if (verbose) {
-    message('Scaling cell embeddings')
-  }
-  embed.mean <- apply(X = cell.embeddings, MARGIN = 1, FUN = mean)
-  embed.sd <- apply(X = cell.embeddings, MARGIN = 1, FUN = sd)
-  norm.embeddings <- (cell.embeddings - embed.mean) / embed.sd
-  if (!is.null(x = scale.max)) {
-    norm.embeddings[norm.embeddings > scale.max] <- scale.max
-    norm.embeddings[norm.embeddings < -scale.max] <- -scale.max
-  }
-  rownames(x = feature.loadings) <- rownames(x = object)
-  colnames(x = feature.loadings) <- paste0(reduction.key, 1:n)
-  rownames(x = norm.embeddings) <- colnames(x = object)
-  colnames(x = norm.embeddings) <- paste0(reduction.key, 1:n)
-  reduction.data <- CreateDimReducObject(
-    embeddings = norm.embeddings,
-    loadings = feature.loadings,
-    assay = assay,
-    stdev = sdev,
-    key = reduction.key
-  )
-  return(reduction.data)
-}
-
-#' @param features Which features to use. If NULL, use variable features
-#'
-#' @rdname RunLSI
-#' @export
-#' @method RunLSI Assay
-RunLSI.Assay <- function(
-  object,
-  assay = NULL,
-  features = NULL,
-  n = 50,
-  reduction.key = 'LSI_',
-  scale.max = NULL,
-  verbose = TRUE,
-  ...
-) {
-  features <- features %||% VariableFeatures(object)
-  data.use <- GetAssayData(
-    object = object,
-    slot = 'counts'
-  )[features, ]
-  reduction.data <- RunLSI(
-    object = data.use,
-    assay = assay,
-    n = n,
-    reduction.key = reduction.key,
-    scale.max = scale.max,
-    verbose = verbose,
-    ...
-  )
-  return(reduction.data)
-}
-
-#' @param reduction.name Name for stored dimension reduction object. Default 'lsi'
-#' @examples
-#' lsi <- RunLSI(object = pbmc_small, n = 5)
-#'
-#' @rdname RunLSI
-#'
-#' @export
-#' @method RunLSI Seurat
-RunLSI.Seurat <- function(
-  object,
-  assay = NULL,
-  features = NULL,
-  n = 50,
-  reduction.key = 'LSI_',
-  reduction.name = 'lsi',
-  scale.max = NULL,
-  verbose = TRUE,
-  ...
-) {
-  assay <- assay %||% DefaultAssay(object)
-  assay.data <- GetAssay(object = object, assay = assay)
-  reduction.data <- RunLSI(
-    object = assay.data,
-    assay = assay,
-    features = features,
-    n = n,
-    reduction.key = reduction.key,
-    scale.max = scale.max,
-    verbose = verbose,
-    ...
-  )
-  object[[reduction.name]] <- reduction.data
-  object <- LogSeuratCommand(object = object)
-  return(object)
-}
-
 #' @param assay Name of Assay PCA is being run on
 #' @param npcs Total Number of PCs to compute and store (50 by default)
 #' @param rev.pca By default computes the PCA on the cell x gene matrix. Setting
@@ -783,6 +837,7 @@ RunLSI.Seurat <- function(
 #' @importFrom utils capture.output
 #'
 #' @rdname RunPCA
+#' @concept dimensional_reduction
 #' @export
 #'
 RunPCA.default <- function(
@@ -832,9 +887,9 @@ RunPCA.default <- function(
       feature.loadings <- pca.results$rotation
       sdev <- pca.results$sdev
       if (weight.by.var) {
-        cell.embeddings <- pca.results$x %*% diag(pca.results$sdev[1:npcs]^2)
-      } else {
         cell.embeddings <- pca.results$x
+      } else {
+        cell.embeddings <- pca.results$x / (pca.results$sdev[1:npcs] * sqrt(x = ncol(x = object) - 1))
       }
     }
   }
@@ -867,6 +922,7 @@ RunPCA.default <- function(
 #' will be dropped, and the PCA will be run using the remaining features.
 #'
 #' @rdname RunPCA
+#' @concept dimensional_reduction
 #' @export
 #' @method RunPCA Assay
 #'
@@ -909,6 +965,7 @@ RunPCA.Assay <- function(
 #' @param reduction.name dimensional reduction name,  pca by default
 #'
 #' @rdname RunPCA
+#' @concept dimensional_reduction
 #' @export
 #' @method RunPCA Seurat
 #'
@@ -958,17 +1015,14 @@ RunPCA.Seurat <- function(
 #' \item{FIt-SNE: }{Use the FFT-accelerated Interpolation-based t-SNE. Based on
 #' Kluger Lab code found here: https://github.com/KlugerLab/FIt-SNE}
 #' }
-#' @param add.iter If an existing tSNE has already been computed, uses the
-#' current tSNE to seed the algorithm and then adds additional iterations on top
-#' of this
 #' @param dim.embed The dimensional space of the resulting tSNE embedding
 #' (default is 2). For example, set to 3 for a 3d tSNE
 #' @param reduction.key dimensional reduction key, specifies the string before the number for the dimension names. tSNE_ by default
 #'
-#' @importFrom tsne tsne
 #' @importFrom Rtsne Rtsne
 #'
 #' @rdname RunTSNE
+#' @concept dimensional_reduction
 #' @export
 #' @method RunTSNE matrix
 #'
@@ -977,7 +1031,6 @@ RunTSNE.matrix <- function(
   assay = NULL,
   seed.use = 1,
   tsne.method = "Rtsne",
-  add.iter = 0,
   dim.embed = 2,
   reduction.key = "tSNE_",
   ...
@@ -990,19 +1043,12 @@ RunTSNE.matrix <- function(
     'Rtsne' = Rtsne(
       X = object,
       dims = dim.embed,
+      pca = FALSE,
       ... # PCA/is_distance
     )$Y,
     'FIt-SNE' = fftRtsne(X = object, dims = dim.embed, rand_seed = seed.use, ...),
     stop("Invalid tSNE method: please choose from 'Rtsne' or 'FIt-SNE'")
   )
-  if (add.iter > 0) {
-    tsne.data <- tsne(
-      X = object,
-      initial_config = as.matrix(x = tsne.data),
-      max_iter = add.iter,
-      ...
-    )
-  }
   colnames(x = tsne.data) <- paste0(reduction.key, 1:ncol(x = tsne.data))
   rownames(x = tsne.data) <- rownames(x = object)
   tsne.reduction <- CreateDimReducObject(
@@ -1018,6 +1064,7 @@ RunTSNE.matrix <- function(
 #' @param dims Which dimensions to use as input features
 #'
 #' @rdname RunTSNE
+#' @concept dimensional_reduction
 #' @export
 #' @method RunTSNE DimReduc
 #'
@@ -1027,7 +1074,6 @@ RunTSNE.DimReduc <- function(
   dims = 1:5,
   seed.use = 1,
   tsne.method = "Rtsne",
-  add.iter = 0,
   dim.embed = 2,
   reduction.key = "tSNE_",
   ...
@@ -1042,6 +1088,7 @@ RunTSNE.DimReduc <- function(
 }
 
 #' @rdname RunTSNE
+#' @concept dimensional_reduction
 #' @export
 #' @method RunTSNE dist
 #'
@@ -1050,7 +1097,6 @@ RunTSNE.dist <- function(
   assay = NULL,
   seed.use = 1,
   tsne.method = "Rtsne",
-  add.iter = 0,
   dim.embed = 2,
   reduction.key = "tSNE_",
   ...
@@ -1072,6 +1118,7 @@ RunTSNE.dist <- function(
 #' @param reduction.name dimensional reduction name, specifies the position in the object$dr list. tsne by default
 #'
 #' @rdname RunTSNE
+#' @concept dimensional_reduction
 #' @export
 #' @method RunTSNE Seurat
 #'
@@ -1083,7 +1130,6 @@ RunTSNE.Seurat <- function(
   features = NULL,
   seed.use = 1,
   tsne.method = "Rtsne",
-  add.iter = 0,
   dim.embed = 2,
   distance.matrix = NULL,
   reduction.name = "tsne",
@@ -1097,7 +1143,6 @@ RunTSNE.Seurat <- function(
       assay = DefaultAssay(object = object),
       seed.use = seed.use,
       tsne.method = tsne.method,
-      add.iter = add.iter,
       dim.embed = dim.embed,
       reduction.key = reduction.key,
       is_distance = TRUE,
@@ -1110,10 +1155,8 @@ RunTSNE.Seurat <- function(
       dims = dims,
       seed.use = seed.use,
       tsne.method = tsne.method,
-      add.iter = add.iter,
       dim.embed = dim.embed,
       reduction.key = reduction.key,
-      pca = FALSE,
       ...
     )
   } else if (!is.null(x = features)) {
@@ -1122,10 +1165,8 @@ RunTSNE.Seurat <- function(
       assay = DefaultAssay(object = object),
       seed.use = seed.use,
       tsne.method = tsne.method,
-      add.iter = add.iter,
       dim.embed = dim.embed,
       reduction.key = reduction.key,
-      pca = FALSE,
       ...
     )
   } else {
@@ -1137,16 +1178,20 @@ RunTSNE.Seurat <- function(
 }
 
 #' @importFrom reticulate py_module_available py_set_seed import
-#' @importFrom uwot umap
+#' @importFrom uwot umap umap_transform
 #' @importFrom future nbrOfWorkers
 #'
 #' @rdname RunUMAP
+#' @concept dimensional_reduction
 #' @method RunUMAP default
 #' @export
 #'
 RunUMAP.default <- function(
   object,
+  reduction.key = 'UMAP_',
   assay = NULL,
+  reduction.model = NULL,
+  return.model = FALSE,
   umap.method = 'uwot',
   n.neighbors = 30L,
   n.components = 2L,
@@ -1165,7 +1210,6 @@ RunUMAP.default <- function(
   seed.use = 42,
   metric.kwds = NULL,
   angular.rp.forest = FALSE,
-  reduction.key = 'UMAP_',
   verbose = TRUE,
   ...
 ) {
@@ -1183,6 +1227,27 @@ RunUMAP.default <- function(
     )
     options(Seurat.warn.umap.uwot = FALSE)
   }
+  if (umap.method == 'uwot-learn') {
+    warning("'uwot-learn' is deprecated. Set umap.method = 'uwot' and return.model = TRUE")
+    umap.method <- "uwot"
+    return.model <- TRUE
+  }
+  if (return.model) {
+    if (verbose) {
+      message("UMAP will return its model")
+    }
+    umap.method = "uwot"
+  }
+  if (inherits(x = object, what = "Neighbor")) {
+    object <- list( idx = Indices(object),
+                    dist = Distances(object) )
+  }
+  if (!is.null(x = reduction.model)) {
+    if (verbose) {
+      message("Running UMAP projection")
+    }
+    umap.method <- "uwot-predict"
+  }
   umap.output <- switch(
     EXPR = umap.method,
     'umap-learn' = {
@@ -1225,31 +1290,106 @@ RunUMAP.default <- function(
         )
         metric <- 'cosine'
       }
-      umap(
-        X = object,
-        n_threads = nbrOfWorkers(),
-        n_neighbors = as.integer(x = n.neighbors),
-        n_components = as.integer(x = n.components),
-        metric = metric,
-        n_epochs = n.epochs,
-        learning_rate = learning.rate,
-        min_dist = min.dist,
-        spread = spread,
-        set_op_mix_ratio = set.op.mix.ratio,
-        local_connectivity = local.connectivity,
-        repulsion_strength = repulsion.strength,
-        negative_sample_rate = negative.sample.rate,
-        a = a,
-        b = b,
-        fast_sgd = uwot.sgd,
-        verbose = verbose
+      if (is.list(x = object)) {
+        umap(
+          X = NULL,
+          nn_method = object,
+          n_threads = nbrOfWorkers(),
+          n_components = as.integer(x = n.components),
+          metric = metric,
+          n_epochs = n.epochs,
+          learning_rate = learning.rate,
+          min_dist = min.dist,
+          spread = spread,
+          set_op_mix_ratio = set.op.mix.ratio,
+          local_connectivity = local.connectivity,
+          repulsion_strength = repulsion.strength,
+          negative_sample_rate = negative.sample.rate,
+          a = a,
+          b = b,
+          fast_sgd = uwot.sgd,
+          verbose = verbose,
+          ret_model = return.model
+        )
+      } else {
+        umap(
+          X = object,
+          n_threads = nbrOfWorkers(),
+          n_neighbors = as.integer(x = n.neighbors),
+          n_components = as.integer(x = n.components),
+          metric = metric,
+          n_epochs = n.epochs,
+          learning_rate = learning.rate,
+          min_dist = min.dist,
+          spread = spread,
+          set_op_mix_ratio = set.op.mix.ratio,
+          local_connectivity = local.connectivity,
+          repulsion_strength = repulsion.strength,
+          negative_sample_rate = negative.sample.rate,
+          a = a,
+          b = b,
+          fast_sgd = uwot.sgd,
+          verbose = verbose,
+          ret_model = return.model
+        )
+      }
+    },
+    'uwot-predict' = {
+      if (metric == 'correlation') {
+        warning(
+          "UWOT does not implement the correlation metric, using cosine instead",
+          call. = FALSE,
+          immediate. = TRUE
+        )
+        metric <- 'cosine'
+      }
+      if (is.null(x = reduction.model) || !inherits(x = reduction.model, what = 'DimReduc')) {
+        stop(
+          "If running projection UMAP, please pass a DimReduc object with the model stored to reduction.model.",
+          call. = FALSE
+        )
+      }
+      model <- Misc(
+        object = reduction.model,
+        slot = "model"
       )
+      if (length(x = model) == 0) {
+        stop(
+          "The provided reduction.model does not have a model stored. Please try running umot-learn on the object first",
+          call. = FALSE
+        )
+      }
+      if (is.list(x = object)) {
+        uwot::umap_transform(
+          X = NULL,
+          nn_method = object,
+          model = model,
+          n_threads = nbrOfWorkers(),
+          n_epochs = n.epochs,
+          verbose = verbose
+        )
+      } else {
+        umap_transform(
+          X = object,
+          model = model,
+          n_threads = nbrOfWorkers(),
+          n_epochs = n.epochs,
+          verbose = verbose
+        )
+      }
     },
     stop("Unknown umap method: ", umap.method, call. = FALSE)
   )
+  if (return.model) {
+    umap.output$nn_index <- NULL
+    umap.model <- umap.output
+    umap.output <- umap.output$embedding
+  }
   colnames(x = umap.output) <- paste0(reduction.key, 1:ncol(x = umap.output))
   if (inherits(x = object, what = 'dist')) {
     rownames(x = umap.output) <- attr(x = object, "Labels")
+  } else if (is.list(x = object)) {
+    rownames(x = umap.output) <- rownames(x = object$idx)
   } else {
     rownames(x = umap.output) <- rownames(x = object)
   }
@@ -1259,12 +1399,16 @@ RunUMAP.default <- function(
     assay = assay,
     global = TRUE
   )
+  if (return.model) {
+    Misc(umap.reduction, slot = "model") <- umap.model
+  }
   return(umap.reduction)
 }
 
 #' @importFrom reticulate py_module_available import
 #'
 #' @rdname RunUMAP
+#' @concept dimensional_reduction
 #' @method RunUMAP Graph
 #' @export
 #'
@@ -1350,6 +1494,27 @@ RunUMAP.Graph <- function(
   return(umap)
 }
 
+#' @rdname RunUMAP
+#' @concept dimensional_reduction
+#' @method RunUMAP Neighbor
+#' @export
+#'
+RunUMAP.Neighbor <- function(
+  object,
+  reduction.model,
+  ...
+) {
+  neighborlist <- list("idx" = Indices(object),
+                       "dist" = Distances(object))
+  RunUMAP(
+    object = neighborlist,
+    reduction.model = reduction.model,
+    ...
+  )
+}
+
+
+#' @param reduction.model \code{DimReduc} object that contains the umap model
 #' @param dims Which dimensions to use as input features, used only if
 #' \code{features} is NULL
 #' @param reduction Which dimensional reduction (PCA or ICA) to use for the
@@ -1360,9 +1525,12 @@ RunUMAP.Graph <- function(
 #' @param graph Name of graph on which to run UMAP
 #' @param assay Assay to pull data for when using \code{features}, or assay used to construct Graph
 #' if running UMAP on a Graph
+#' @param nn.name Name of knn output on which to run UMAP
+#' @param slot The slot used to pull data for when using \code{features}. data slot is by default.
 #' @param umap.method UMAP implementation to run. Can be
 #' \describe{
 #'   \item{\code{uwot}:}{Runs umap via the uwot R package}
+#'   \item{\code{uwot-learn}:}{Runs umap via the uwot R package and return the learned umap model}
 #'   \item{\code{umap-learn}:}{Run the Seurat wrapper of the python umap-learn package}
 #' }
 #' @param n.neighbors This determines the number of neighboring points used in
@@ -1413,11 +1581,13 @@ RunUMAP.Graph <- function(
 #' @param reduction.name Name to store dimensional reduction under in the Seurat object
 #' @param reduction.key dimensional reduction key, specifies the string before
 #' the number for the dimension names. UMAP by default
+#' @param return.model whether UMAP will return the uwot model
 #' @param seed.use Set a random seed. By default, sets the seed to 42. Setting
 #' NULL will not set a seed
 #' @param verbose Controls verbosity
 #'
 #' @rdname RunUMAP
+#' @concept dimensional_reduction
 #' @export
 #' @method RunUMAP Seurat
 #'
@@ -1427,8 +1597,12 @@ RunUMAP.Seurat <- function(
   reduction = 'pca',
   features = NULL,
   graph = NULL,
-  assay = 'RNA',
+  assay = DefaultAssay(object = object),
+  nn.name = NULL,
+  slot = 'data',
   umap.method = 'uwot',
+  reduction.model = NULL,
+  return.model = FALSE,
   n.neighbors = 30L,
   n.components = 2L,
   metric = 'cosine',
@@ -1456,7 +1630,7 @@ RunUMAP.Seurat <- function(
       stop("Please specify only one of the following arguments: dims, features, or graph")
   }
   if (!is.null(x = features)) {
-    data.use <- as.matrix(x = t(x = GetAssayData(object = object, slot = 'data', assay = assay)[features, , drop = FALSE]))
+    data.use <- as.matrix(x = t(x = GetAssayData(object = object, slot = slot, assay = assay)[features, , drop = FALSE]))
     if (ncol(x = data.use) < n.components) {
       stop(
         "Please provide as many or more features than n.components: ",
@@ -1480,6 +1654,8 @@ RunUMAP.Seurat <- function(
         call. = FALSE
       )
     }
+  }  else if (!is.null( x = nn.name)){
+    data.use <- object[[nn.name]]
   } else if (!is.null(x = graph)) {
     data.use <- object[[graph]]
   } else {
@@ -1487,6 +1663,8 @@ RunUMAP.Seurat <- function(
   }
   object[[reduction.name]] <- RunUMAP(
     object = data.use,
+    reduction.model = reduction.model,
+    return.model = return.model,
     assay = assay,
     umap.method = umap.method,
     n.neighbors = n.neighbors,
@@ -1520,6 +1698,7 @@ RunUMAP.Seurat <- function(
 #' @importFrom stats prop.test
 #'
 #' @rdname ScoreJackStraw
+#' @concept dimensional_reduction
 #' @export
 #' @method ScoreJackStraw JackStrawData
 #'
@@ -1559,6 +1738,7 @@ ScoreJackStraw.JackStrawData <- function(
 }
 
 #' @rdname ScoreJackStraw
+#' @concept dimensional_reduction
 #' @export
 #' @method ScoreJackStraw DimReduc
 #'
@@ -1579,6 +1759,7 @@ ScoreJackStraw.DimReduc <- function(object, dims = 1:5, score.thresh = 1e-5, ...
 #' @seealso \code{\link{JackStrawPlot}}
 #'
 #' @rdname ScoreJackStraw
+#' @concept dimensional_reduction
 #' @export
 #' @method ScoreJackStraw Seurat
 #'
@@ -1674,8 +1855,8 @@ EmpiricalP <- function(x, nullval) {
 
 # FIt-SNE helper function for calling fast_tsne from R
 #
-# Based on Kluger Lab FIt-SNE v1.1.0 code on https://github.com/KlugerLab/FIt-SNE/blob/master/fast_tsne.R
-# commit d2cf403 on Feb 8, 2019
+# Based on Kluger Lab FIt-SNE v1.2.1 code on https://github.com/KlugerLab/FIt-SNE/blob/master/fast_tsne.R
+# commit 601608ed42e4be2765970910927da20f0b0bf9b9 on June 25, 2020
 #
 #' @importFrom utils file_test
 #
@@ -1683,19 +1864,18 @@ fftRtsne <- function(X,
   dims = 2,
   perplexity = 30,
   theta = 0.5,
-  check_duplicates = TRUE,
-  max_iter = 1000,
+  max_iter = 750,
   fft_not_bh = TRUE,
   ann_not_vptree = TRUE,
   stop_early_exag_iter = 250,
   exaggeration_factor = 12.0,
   no_momentum_during_exag = FALSE,
-  start_late_exag_iter = -1.0,
+  start_late_exag_iter = -1,
   late_exag_coeff = 1.0,
   mom_switch_iter = 250,
   momentum = 0.5,
   final_momentum = 0.8,
-  learning_rate = 200,
+  learning_rate = 'auto',
   n_trees = 50,
   search_k = -1,
   rand_seed = -1,
@@ -1704,7 +1884,8 @@ fftRtsne <- function(X,
   min_num_intervals = 50,
   K = -1,
   sigma = -30,
-  initialization = NULL,
+  initialization = 'pca',
+  max_step_norm = 5,
   data_path = NULL,
   result_path = NULL,
   load_affinities = NULL,
@@ -1739,19 +1920,19 @@ fftRtsne <- function(X,
   }
   # check fast_tsne version
   ft.out <- suppressWarnings(expr = system2(command = fast_tsne_path, stdout = TRUE))
-  if (grepl(pattern = '= t-SNE v1.1', x = ft.out[1])) {
-    version_number <- '1.1.0'
-  } else if (grepl(pattern = '= t-SNE v1.0', x = ft.out[1])) {
-    version_number <- '1.0'
-  } else {
+  version_number <- regmatches(ft.out[1], regexpr('= t-SNE v[0-9.]+', ft.out[1]))
+  if (is.null(version_number)){
     message("First line of fast_tsne output is")
     message(ft.out[1])
-    stop("Our FIt-SNE wrapper requires FIt-SNE v1.X.X, please install the appropriate version from github.com/KlugerLab/FIt-SNE and have fast_tsne_path point to it if it's not in your path")
+    stop("Our FIt-SNE wrapper requires FIt-SNE v1.0+, please install the appropriate version from github.com/KlugerLab/FIt-SNE and have fast_tsne_path point to it if it's not in your path")
+  } else {
+    version_number <- gsub('= t-SNE v', '', version_number)
   }
+
   is.wholenumber <- function(x, tol = .Machine$double.eps ^ 0.5) {
     return(abs(x = x - round(x = x)) < tol)
   }
-  if (version_number == '1.0' && df != 1.0) {
+  if (version_number == '1.0.0' && df != 1.0) {
     stop("This version of FIt-SNE does not support df!=1. Please install the appropriate version from github.com/KlugerLab/FIt-SNE")
   }
   if (!is.numeric(x = theta) || (theta < 0.0) || (theta > 1.0) ) {
@@ -1772,6 +1953,9 @@ fftRtsne <- function(X,
   if (!is.numeric(x = exaggeration_factor)) {
     stop("exaggeration_factor should be numeric")
   }
+  if (!is.numeric(df)) {
+    stop("df should be numeric")
+  }
   if (!is.wholenumber(x = dims) || dims <= 0) {
     stop("Incorrect dimensionality.")
   }
@@ -1781,10 +1965,54 @@ fftRtsne <- function(X,
     } else if (perplexity == 0) {
       search_k <- n_trees * max(perplexity_list) * 3
     } else {
-      search_k <- n_trees * K * 3
+      search_k <- n_trees * K
     }
   }
+
+  if (is.character(learning_rate) && learning_rate =='auto') {
+    learning_rate = max(200, nrow(X)/exaggeration_factor)
+  }
+  if (is.character(start_late_exag_iter) && start_late_exag_iter =='auto') {
+    if (late_exag_coeff > 0) {
+      start_late_exag_iter = stop_early_exag_iter
+    } else {
+      start_late_exag_iter = -1
+    }
+  }
+
+  if (is.character(initialization) && initialization == 'pca') {
+    if (rand_seed != -1)  {
+      set.seed(rand_seed)
+    }
+    if (requireNamespace("rsvd", quietly = TRUE)) {
+      message('Using rsvd() to compute the top PCs for initialization.')
+      X_c <- scale(x = X, center = TRUE, scale = FALSE)
+      rsvd_out <- rsvd::rsvd(A = X_c, k = dims)
+      X_top_pcs <- rsvd_out$u %*% diag(x = rsvd_out$d, nrow = dims)
+    } else if (requireNamespace("irlba", quietly = TRUE)) {
+      message('Using irlba() to compute the top PCs for initialization.')
+      X_colmeans <- colMeans(x = X)
+      irlba_out <- irlba::irlba(A = X, nv = dims, center = X_colmeans)
+      X_top_pcs <- irlba_out$u %*% diag(x = irlba_out$d, nrow = dims)
+    } else {
+      stop(
+        "By default, FIt-SNE initializes the embedding with the
+        top PCs. We use either rsvd or irlba for fast computation.
+        To use this functionality, please install the rsvd package
+        with install.packages('rsvd') or the irlba package with
+        install.packages('ilrba').  Otherwise, set initialization
+        to NULL for random initialization, or any N by dims matrix
+        for custom initialization."
+      )
+    }
+    initialization <- 0.0001*(X_top_pcs/sd(X_top_pcs[,1]))
+
+  } else if (is.character(x = initialization) && initialization == 'random') {
+    message('Random initialization')
+    initialization = NULL
+  }
   nbody_algo <- ifelse(test = fft_not_bh, yes = 2, no = 1)
+
   if (is.null(load_affinities)) {
     load_affinities <- 0
   } else {
@@ -1796,18 +2024,23 @@ fftRtsne <- function(X,
       load_affinities <- 0
     }
   }
+
   knn_algo <- ifelse(test = ann_not_vptree, yes = 1, no = 2)
+  tX <- as.numeric(t(X))
+
   f <- file(description = data_path, open = "wb")
   n = nrow(x = X)
   D = ncol(x = X)
   writeBin(object = as.integer(x = n), con = f, size = 4)
   writeBin(object = as.integer(x = D), con = f, size = 4)
-  writeBin(object = as.numeric(x = theta), con = f, size = 8) #theta
-  writeBin(object = as.numeric(x = perplexity), con = f, size = 8) #theta
+  writeBin(object = as.numeric(x = theta), con = f, size = 8)
+  writeBin(object = as.numeric(x = perplexity), con = f, size = 8)
+
   if (perplexity == 0) {
     writeBin(object = as.integer(x = length(x = perplexity_list)), con = f, size = 4)
     writeBin(object = perplexity_list, con = f)
   }
+
   writeBin(object = as.integer(x = dims), con = f, size = 4) #theta
   writeBin(object = as.integer(x = max_iter), con = f, size = 4)
   writeBin(object = as.integer(x = stop_early_exag_iter), con = f, size = 4)
@@ -1815,6 +2048,9 @@ fftRtsne <- function(X,
   writeBin(object = as.numeric(x = momentum), con = f, size = 8)
   writeBin(object = as.numeric(x = final_momentum), con = f, size = 8)
   writeBin(object = as.numeric(x = learning_rate), con = f, size = 8)
+  if (!(version_number %in% c('1.1.0', '1.0.0'))) {
+    writeBin(object = as.numeric(x = max_step_norm), f, size = 8)
+  }
   writeBin(object = as.integer(x = K), con = f, size = 4) #K
   writeBin(object = as.numeric(x = sigma), con = f, size = 8) #sigma
   writeBin(object = as.integer(x = nbody_algo), con = f, size = 4)  #not barnes hut
@@ -1828,10 +2064,9 @@ fftRtsne <- function(X,
   writeBin(object = as.integer(x = nterms), con = f, size = 4)
   writeBin(object = as.numeric(x = intervals_per_integer), con = f, size = 8)
   writeBin(object = as.integer(x = min_num_intervals), con = f, size = 4)
-  tX = c(t(X))
   writeBin(object = tX, con = f)
   writeBin(object = as.integer(x = rand_seed), con = f, size = 4)
-  if (version_number != "1.0") {
+  if (version_number != "1.0.0") {
     writeBin(object = as.numeric(x = df), con = f, size = 8)
   }
   writeBin(object = as.integer(x = load_affinities), con = f, size = 4)
@@ -1839,10 +2074,11 @@ fftRtsne <- function(X,
     writeBin(object = c(t(x = initialization)), con = f)
   }
   close(con = f)
-  if (version_number == "1.0") {
+
+  if (version_number == "1.0.0") {
     flag <- system2(
-     command = fast_tsne_path,
-     args = c(data_path, result_path, nthreads)
+      command = fast_tsne_path,
+      args = c(data_path, result_path, nthreads)
     )
   } else {
     flag <- system2(
@@ -1850,9 +2086,11 @@ fftRtsne <- function(X,
       args = c(version_number, data_path, result_path, nthreads)
     )
   }
+
   if (flag != 0) {
     stop('tsne call failed')
   }
+
   f <- file(description = result_path, open = "rb")
   n <- readBin(con = f, what = integer(), n = 1, size = 4)
   d <- readBin(con = f, what = integer(), n = 1, size = 4)
@@ -1972,3 +2210,129 @@ PrepDR <- function(
   data.use <- data.use[features, ]
   return(data.use)
 }
+
+#' @param assay Name of Assay SPCA is being run on
+#' @param npcs Total Number of SPCs to compute and store (50 by default)
+#' @param verbose Print the top genes associated with high/low loadings for
+#' the SPCs
+#' @param reduction.key dimensional reduction key, specifies the string before
+#' the number for the dimension names. SPC by default
+#' @param graph Graph used supervised by SPCA
+#' @param seed.use Set a random seed. By default, sets the seed to 42. Setting
+#' NULL will not set a seed.
+#'
+#' @importFrom irlba irlba
+#'
+#' @rdname RunSPCA
+#' @export
+RunSPCA.default <- function(
+  object,
+  assay = NULL,
+  npcs = 50,
+  reduction.key = "SPC_",
+  graph = NULL,
+  verbose = FALSE,
+  seed.use = 42,
+  ...
+) {
+  if (!is.null(x = seed.use)) {
+    set.seed(seed = seed.use)
+  }
+  npcs <- min(npcs, nrow(x = object) - 1)
+  if (verbose) {
+    message("Computing sPCA transformation")
+  }
+  HSIC <- object %*% graph %*% t(x = object)
+  pca.results <- irlba(A = HSIC, nv = npcs)
+  feature.loadings <- pca.results$u
+  rownames(x = feature.loadings) <- rownames(x = object)
+  cell.embeddings <- t(object) %*% feature.loadings
+  colnames(x = cell.embeddings) <- colnames(x = feature.loadings) <-
+    paste0(reduction.key, 1:ncol(x = cell.embeddings))
+  sdev <- pca.results$d / sqrt(max(1, nrow(x = HSIC) - 1))
+  reduction.data <- CreateDimReducObject(
+    embeddings = cell.embeddings,
+    loadings = feature.loadings,
+    assay = assay,
+    stdev = sdev,
+    key = reduction.key
+  )
+  return(reduction.data)
+}
+
+#' @param features Features to compute SPCA on. If features=NULL, SPCA will be run
+#' using the variable features for the Assay.
+#'
+#' @rdname RunSPCA
+#' @export
+#' @method RunSPCA Assay
+#'
+RunSPCA.Assay <- function(
+  object,
+  assay = NULL,
+  features = NULL,
+  npcs = 50,
+  reduction.key = "SPC_",
+  graph = NULL,
+  verbose = TRUE,
+  seed.use = 42,
+  ...
+) {
+  data.use <- PrepDR(
+    object = object,
+    features = features,
+    verbose = verbose
+  )
+  reduction.data <- RunSPCA(
+    object = data.use,
+    assay = assay,
+    npcs = npcs,
+    reduction.key = reduction.key,
+    graph = graph,
+    verbose = verbose,
+    seed.use = seed.use,
+    ...
+  )
+  return(reduction.data)
+}
+
+#' @param reduction.name dimensional reduction name, spca by default
+#' @rdname RunSPCA
+#' @export
+#' @method RunSPCA Seurat
+#'
+RunSPCA.Seurat <- function(
+  object,
+  assay = NULL,
+  features = NULL,
+  npcs = 50,
+  reduction.name = "spca",
+  reduction.key = "SPC_",
+  graph = NULL,
+  verbose = TRUE,
+  seed.use = 42,
+  ...
+) {
+  assay <- assay %||% DefaultAssay(object = object)
+  assay.data <- GetAssay(object = object, assay = assay)
+  if (is.null(x = graph)) {
+    stop("Graph is not provided")
+  } else if (is.character(x = graph)) {
+    graph <- object[[graph]]
+  }
+  reduction.data <- RunSPCA(
+    object = assay.data,
+    assay = assay,
+    features = features,
+    npcs = npcs,
+    reduction.name = reduction.name,
+    reduction.key = reduction.key,
+    graph = graph,
+    verbose = verbose,
+    seed.use = seed.use,
+    ...
+  )
+  object[[reduction.name]] <- reduction.data
+  object <- LogSeuratCommand(object = object)
+  return(object)
+}
